"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class SlackChannelEndpointDtoTypedDict(TypedDict):
    channel_id: str
    r"""Slack channel ID"""


class SlackChannelEndpointDto(BaseModel):
    channel_id: Annotated[str, pydantic.Field(alias="channelId")]
    r"""Slack channel ID"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutresponsedto import LayoutResponseDto, LayoutResponseDtoTypedDict
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class ListLayoutResponseDtoTypedDict(TypedDict):
    layouts: List[LayoutResponseDtoTypedDict]
    r"""List of layouts"""
    total_count: float
    r"""Total number of layouts"""


class ListLayoutResponseDto(BaseModel):
    layouts: List[LayoutResponseDto]
    r"""List of layouts"""

    total_count: Annotated[float, pydantic.Field(alias="totalCount")]
    r"""Total number of layouts"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailcontrolsdto import EmailControlsDto, EmailControlsDtoTypedDict
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ControlValuesTypedDict(TypedDict):
    r"""Control values for the layout"""

    email: NotRequired[EmailControlsDtoTypedDict]
    r"""Email layout controls"""


class ControlValues(BaseModel):
    r"""Control values for the layout"""

    email: Optional[EmailControlsDto] = None
    r"""Email layout controls"""


class UpdateLayoutDtoTypedDict(TypedDict):
    name: str
    r"""Name of the layout"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this layout"""
    control_values: NotRequired[Nullable[ControlValuesTypedDict]]
    r"""Control values for the layout"""


class UpdateLayoutDto(BaseModel):
    name: str
    r"""Name of the layout"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this layout"""

    control_values: Annotated[
        OptionalNullable[ControlValues], pydantic.Field(alias="controlValues")
    ] = UNSET
    r"""Control values for the layout"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["isTranslationEnabled", "controlValues"]
        nullable_fields = ["controlValues"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TranslationGroupDtoResourceType(str, Enum):
    r"""Resource type"""

    WORKFLOW = "workflow"
    LAYOUT = "layout"


class TranslationGroupDtoTypedDict(TypedDict):
    resource_id: str
    r"""Resource identifier (slugified ID)"""
    resource_type: TranslationGroupDtoResourceType
    r"""Resource type"""
    resource_name: str
    r"""Resource name (e.g., workflow name)"""
    locales: List[str]
    r"""Array of available locales for this resource"""
    created_at: str
    r"""Creation timestamp"""
    updated_at: str
    r"""Last update timestamp"""
    outdated_locales: NotRequired[List[str]]
    r"""Locales that are outdated compared to the default locale (only present when there are outdated locales)"""


class TranslationGroupDto(BaseModel):
    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""Resource identifier (slugified ID)"""

    resource_type: Annotated[
        TranslationGroupDtoResourceType, pydantic.Field(alias="resourceType")
    ]
    r"""Resource type"""

    resource_name: Annotated[str, pydantic.Field(alias="resourceName")]
    r"""Resource name (e.g., workflow name)"""

    locales: List[str]
    r"""Array of available locales for this resource"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""Creation timestamp"""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""Last update timestamp"""

    outdated_locales: Annotated[
        Optional[List[str]], pydantic.Field(alias="outdatedLocales")
    ] = None
    r"""Locales that are outdated compared to the default locale (only present when there are outdated locales)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activitynotificationexecutiondetailresponsedto import (
    ActivityNotificationExecutionDetailResponseDto,
    ActivityNotificationExecutionDetailResponseDtoTypedDict,
)
from .activitynotificationstepresponsedto import (
    ActivityNotificationStepResponseDto,
    ActivityNotificationStepResponseDtoTypedDict,
)
from .digestmetadatadto import DigestMetadataDto, DigestMetadataDtoTypedDict
from .providersidenum import ProvidersIDEnum
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivityNotificationJobResponseDtoType(str, Enum):
    r"""Type of the job"""

    IN_APP = "in_app"
    EMAIL = "email"
    SMS = "sms"
    CHAT = "chat"
    PUSH = "push"
    DIGEST = "digest"
    TRIGGER = "trigger"
    DELAY = "delay"
    THROTTLE = "throttle"
    CUSTOM = "custom"


class ActivityNotificationJobResponseDtoPayloadTypedDict(TypedDict):
    r"""Optional payload for the job"""


class ActivityNotificationJobResponseDtoPayload(BaseModel):
    r"""Optional payload for the job"""


class ActivityNotificationJobResponseDtoTypedDict(TypedDict):
    id: str
    r"""Unique identifier of the job"""
    type: ActivityNotificationJobResponseDtoType
    r"""Type of the job"""
    execution_details: List[ActivityNotificationExecutionDetailResponseDtoTypedDict]
    r"""Execution details of the job"""
    step: ActivityNotificationStepResponseDtoTypedDict
    r"""Step details of the job"""
    provider_id: ProvidersIDEnum
    r"""Provider ID of the job"""
    status: str
    r"""Status of the job"""
    digest: NotRequired[DigestMetadataDtoTypedDict]
    r"""Optional digest for the job, including metadata and events"""
    overrides: NotRequired[Dict[str, Any]]
    r"""Optional context object for additional error details."""
    payload: NotRequired[ActivityNotificationJobResponseDtoPayloadTypedDict]
    r"""Optional payload for the job"""
    updated_at: NotRequired[str]
    r"""Updated time of the notification"""
    schedule_extensions_count: NotRequired[float]
    r"""The number of times the digest/delay job has been extended to align with the subscribers schedule"""


class ActivityNotificationJobResponseDto(BaseModel):
    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Unique identifier of the job"""

    type: ActivityNotificationJobResponseDtoType
    r"""Type of the job"""

    execution_details: Annotated[
        List[ActivityNotificationExecutionDetailResponseDto],
        pydantic.Field(alias="executionDetails"),
    ]
    r"""Execution details of the job"""

    step: ActivityNotificationStepResponseDto
    r"""Step details of the job"""

    provider_id: Annotated[ProvidersIDEnum, pydantic.Field(alias="providerId")]
    r"""Provider ID of the job"""

    status: str
    r"""Status of the job"""

    digest: Optional[DigestMetadataDto] = None
    r"""Optional digest for the job, including metadata and events"""

    overrides: Optional[Dict[str, Any]] = None
    r"""Optional context object for additional error details."""

    payload: Optional[ActivityNotificationJobResponseDtoPayload] = None
    r"""Optional payload for the job"""

    updated_at: Annotated[Optional[str], pydantic.Field(alias="updatedAt")] = None
    r"""Updated time of the notification"""

    schedule_extensions_count: Annotated[
        Optional[float], pydantic.Field(alias="scheduleExtensionsCount")
    ] = None
    r"""The number of times the digest/delay job has been extended to align with the subscribers schedule"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrdinalEnum(str, Enum):
    r"""Ordinal position for the digest"""

    ONE = "1"
    TWO = "2"
    THREE = "3"
    FOUR = "4"
    FIVE = "5"
    LAST = "last"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DuplicateLayoutDtoTypedDict(TypedDict):
    name: str
    r"""Name of the layout"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this layout"""


class DuplicateLayoutDto(BaseModel):
    name: str
    r"""Name of the layout"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this layout"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DataTypedDict(TypedDict):
    pass


class Data(BaseModel):
    pass


class TenantPayloadDtoTypedDict(TypedDict):
    identifier: NotRequired[str]
    name: NotRequired[str]
    data: NotRequired[DataTypedDict]


class TenantPayloadDto(BaseModel):
    identifier: Optional[str] = None

    name: Optional[str] = None

    data: Optional[Data] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatstepresponsedto import ChatStepResponseDto, ChatStepResponseDtoTypedDict
from .customstepresponsedto import CustomStepResponseDto, CustomStepResponseDtoTypedDict
from .delaystepresponsedto import DelayStepResponseDto, DelayStepResponseDtoTypedDict
from .digeststepresponsedto import DigestStepResponseDto, DigestStepResponseDtoTypedDict
from .emailstepresponsedto import EmailStepResponseDto, EmailStepResponseDtoTypedDict
from .inappstepresponsedto import InAppStepResponseDto, InAppStepResponseDtoTypedDict
from .pushstepresponsedto import PushStepResponseDto, PushStepResponseDtoTypedDict
from .resourceoriginenum import ResourceOriginEnum
from .runtimeissuedto import RuntimeIssueDto, RuntimeIssueDtoTypedDict
from .severitylevelenum import SeverityLevelEnum
from .smsstepresponsedto import SmsStepResponseDto, SmsStepResponseDtoTypedDict
from .throttlestepresponsedto import (
    ThrottleStepResponseDto,
    ThrottleStepResponseDtoTypedDict,
)
from .workflowpreferencesresponsedto import (
    WorkflowPreferencesResponseDto,
    WorkflowPreferencesResponseDtoTypedDict,
)
from .workflowstatusenum import WorkflowStatusEnum
from novu_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from novu_py.utils import get_discriminator
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class WorkflowResponseDtoUpdatedByTypedDict(TypedDict):
    r"""User who last updated the workflow"""

    id: str
    r"""User ID"""
    first_name: NotRequired[Nullable[str]]
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[Nullable[str]]
    r"""User external ID"""


class WorkflowResponseDtoUpdatedBy(BaseModel):
    r"""User who last updated the workflow"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["firstName", "lastName", "externalId"]
        nullable_fields = ["firstName", "lastName", "externalId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class LastPublishedByTypedDict(TypedDict):
    r"""User who last published the workflow"""

    id: str
    r"""User ID"""
    first_name: NotRequired[Nullable[str]]
    r"""User first name"""
    last_name: NotRequired[Nullable[str]]
    r"""User last name"""
    external_id: NotRequired[Nullable[str]]
    r"""User external ID"""


class LastPublishedBy(BaseModel):
    r"""User who last published the workflow"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""User ID"""

    first_name: Annotated[OptionalNullable[str], pydantic.Field(alias="firstName")] = (
        UNSET
    )
    r"""User first name"""

    last_name: Annotated[OptionalNullable[str], pydantic.Field(alias="lastName")] = (
        UNSET
    )
    r"""User last name"""

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET
    r"""User external ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["firstName", "lastName", "externalId"]
        nullable_fields = ["firstName", "lastName", "externalId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


WorkflowResponseDtoStepsTypedDict = TypeAliasType(
    "WorkflowResponseDtoStepsTypedDict",
    Union[
        InAppStepResponseDtoTypedDict,
        EmailStepResponseDtoTypedDict,
        SmsStepResponseDtoTypedDict,
        PushStepResponseDtoTypedDict,
        ChatStepResponseDtoTypedDict,
        DelayStepResponseDtoTypedDict,
        DigestStepResponseDtoTypedDict,
        CustomStepResponseDtoTypedDict,
        ThrottleStepResponseDtoTypedDict,
    ],
)


WorkflowResponseDtoSteps = Annotated[
    Union[
        Annotated[InAppStepResponseDto, Tag("in_app")],
        Annotated[EmailStepResponseDto, Tag("email")],
        Annotated[SmsStepResponseDto, Tag("sms")],
        Annotated[PushStepResponseDto, Tag("push")],
        Annotated[ChatStepResponseDto, Tag("chat")],
        Annotated[DelayStepResponseDto, Tag("delay")],
        Annotated[DigestStepResponseDto, Tag("digest")],
        Annotated[CustomStepResponseDto, Tag("custom")],
        Annotated[ThrottleStepResponseDto, Tag("throttle")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class WorkflowResponseDtoTypedDict(TypedDict):
    name: str
    r"""Name of the workflow"""
    id: str
    r"""Database identifier of the workflow"""
    workflow_id: str
    r"""Workflow identifier"""
    slug: str
    r"""Slug of the workflow"""
    updated_at: str
    r"""Last updated timestamp"""
    created_at: str
    r"""Creation timestamp"""
    steps: List[WorkflowResponseDtoStepsTypedDict]
    r"""Steps of the workflow"""
    origin: ResourceOriginEnum
    r"""Origin of the layout"""
    preferences: WorkflowPreferencesResponseDtoTypedDict
    r"""Preferences for the workflow"""
    status: WorkflowStatusEnum
    r"""Status of the workflow"""
    severity: SeverityLevelEnum
    r"""Severity of the workflow"""
    description: NotRequired[str]
    r"""Description of the workflow"""
    tags: NotRequired[List[str]]
    r"""Tags associated with the workflow"""
    active: NotRequired[bool]
    r"""Whether the workflow is active"""
    validate_payload: NotRequired[bool]
    r"""Enable or disable payload schema validation"""
    payload_schema: NotRequired[Nullable[Dict[str, Any]]]
    r"""The payload JSON Schema for the workflow"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this workflow"""
    updated_by: NotRequired[Nullable[WorkflowResponseDtoUpdatedByTypedDict]]
    r"""User who last updated the workflow"""
    last_published_at: NotRequired[Nullable[str]]
    r"""Timestamp of the last workflow publication"""
    last_published_by: NotRequired[Nullable[LastPublishedByTypedDict]]
    r"""User who last published the workflow"""
    issues: NotRequired[Dict[str, RuntimeIssueDtoTypedDict]]
    r"""Runtime issues for workflow creation and update"""
    last_triggered_at: NotRequired[Nullable[str]]
    r"""Timestamp of the last workflow trigger"""
    payload_example: NotRequired[Nullable[Dict[str, Any]]]
    r"""Generated payload example based on the payload schema"""


class WorkflowResponseDto(BaseModel):
    name: str
    r"""Name of the workflow"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Database identifier of the workflow"""

    workflow_id: Annotated[str, pydantic.Field(alias="workflowId")]
    r"""Workflow identifier"""

    slug: str
    r"""Slug of the workflow"""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""Last updated timestamp"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""Creation timestamp"""

    steps: List[WorkflowResponseDtoSteps]
    r"""Steps of the workflow"""

    origin: ResourceOriginEnum
    r"""Origin of the layout"""

    preferences: WorkflowPreferencesResponseDto
    r"""Preferences for the workflow"""

    status: WorkflowStatusEnum
    r"""Status of the workflow"""

    severity: SeverityLevelEnum
    r"""Severity of the workflow"""

    description: Optional[str] = None
    r"""Description of the workflow"""

    tags: Optional[List[str]] = None
    r"""Tags associated with the workflow"""

    active: Optional[bool] = False
    r"""Whether the workflow is active"""

    validate_payload: Annotated[
        Optional[bool], pydantic.Field(alias="validatePayload")
    ] = None
    r"""Enable or disable payload schema validation"""

    payload_schema: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="payloadSchema")
    ] = UNSET
    r"""The payload JSON Schema for the workflow"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this workflow"""

    updated_by: Annotated[
        OptionalNullable[WorkflowResponseDtoUpdatedBy],
        pydantic.Field(alias="updatedBy"),
    ] = UNSET
    r"""User who last updated the workflow"""

    last_published_at: Annotated[
        OptionalNullable[str], pydantic.Field(alias="lastPublishedAt")
    ] = UNSET
    r"""Timestamp of the last workflow publication"""

    last_published_by: Annotated[
        OptionalNullable[LastPublishedBy], pydantic.Field(alias="lastPublishedBy")
    ] = UNSET
    r"""User who last published the workflow"""

    issues: Optional[Dict[str, RuntimeIssueDto]] = None
    r"""Runtime issues for workflow creation and update"""

    last_triggered_at: Annotated[
        OptionalNullable[str], pydantic.Field(alias="lastTriggeredAt")
    ] = UNSET
    r"""Timestamp of the last workflow trigger"""

    payload_example: Annotated[
        OptionalNullable[Dict[str, Any]], pydantic.Field(alias="payloadExample")
    ] = UNSET
    r"""Generated payload example based on the payload schema"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "tags",
            "active",
            "validatePayload",
            "payloadSchema",
            "isTranslationEnabled",
            "updatedBy",
            "lastPublishedAt",
            "lastPublishedBy",
            "issues",
            "lastTriggeredAt",
            "payloadExample",
        ]
        nullable_fields = [
            "payloadSchema",
            "updatedBy",
            "lastPublishedAt",
            "lastPublishedBy",
            "lastTriggeredAt",
            "payloadExample",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .notificationtriggerdto import (
    NotificationTriggerDto,
    NotificationTriggerDtoTypedDict,
)
from .resourceoriginenum import ResourceOriginEnum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivityNotificationTemplateResponseDtoTypedDict(TypedDict):
    name: str
    r"""Name of the template"""
    triggers: List[NotificationTriggerDtoTypedDict]
    r"""Triggers of the template"""
    id: NotRequired[str]
    r"""Unique identifier of the template"""
    origin: NotRequired[ResourceOriginEnum]
    r"""Origin of the layout"""


class ActivityNotificationTemplateResponseDto(BaseModel):
    name: str
    r"""Name of the template"""

    triggers: List[NotificationTriggerDto]
    r"""Triggers of the template"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Unique identifier of the template"""

    origin: Optional[ResourceOriginEnum] = None
    r"""Origin of the layout"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Status(str, Enum):
    r"""Status of the event"""

    OPENED = "opened"
    REJECTED = "rejected"
    SENT = "sent"
    DEFERRED = "deferred"
    DELIVERED = "delivered"
    BOUNCED = "bounced"
    DROPPED = "dropped"
    CLICKED = "clicked"
    BLOCKED = "blocked"
    SPAM = "spam"
    UNSUBSCRIBED = "unsubscribed"
    DELAYED = "delayed"
    COMPLAINT = "complaint"
    CREATED = "created"
    ACCEPTED = "accepted"
    QUEUED = "queued"
    SENDING = "sending"
    FAILED = "failed"
    UNDELIVERED = "undelivered"
    DISMISSED = "dismissed"


class EventBodyTypedDict(TypedDict):
    status: Status
    r"""Status of the event"""
    date_: str
    r"""Date of the event"""
    external_id: NotRequired[str]
    r"""External ID from the provider"""
    attempts: NotRequired[float]
    r"""Number of attempts"""
    response: NotRequired[str]
    r"""Response from the provider"""
    row: NotRequired[str]
    r"""Raw content from the provider webhook"""


class EventBody(BaseModel):
    status: Status
    r"""Status of the event"""

    date_: Annotated[str, pydantic.Field(alias="date")]
    r"""Date of the event"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""External ID from the provider"""

    attempts: Optional[float] = None
    r"""Number of attempts"""

    response: Optional[str] = None
    r"""Response from the provider"""

    row: Optional[str] = None
    r"""Raw content from the provider webhook"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelctatypeenum import ChannelCTATypeEnum
from .messageaction import MessageAction, MessageActionTypedDict
from .messagectadata import MessageCTAData, MessageCTADataTypedDict
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MessageCTATypedDict(TypedDict):
    type: NotRequired[ChannelCTATypeEnum]
    r"""Type of call to action"""
    data: NotRequired[MessageCTADataTypedDict]
    r"""Data associated with the call to action"""
    action: NotRequired[MessageActionTypedDict]
    r"""Action associated with the call to action"""


class MessageCTA(BaseModel):
    type: Optional[ChannelCTATypeEnum] = None
    r"""Type of call to action"""

    data: Optional[MessageCTAData] = None
    r"""Data associated with the call to action"""

    action: Optional[MessageAction] = None
    r"""Action associated with the call to action"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class LayoutCreationSourceEnum(str, Enum):
    r"""Source of layout creation"""

    DASHBOARD = "dashboard"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ActivityNotificationSubscriberResponseDtoTypedDict(TypedDict):
    subscriber_id: str
    r"""External unique identifier of the subscriber"""
    id: str
    r"""Internal to Novu unique identifier of the subscriber"""
    first_name: NotRequired[str]
    r"""First name of the subscriber"""
    last_name: NotRequired[str]
    r"""Last name of the subscriber"""
    email: NotRequired[str]
    r"""Email address of the subscriber"""
    phone: NotRequired[str]
    r"""Phone number of the subscriber"""


class ActivityNotificationSubscriberResponseDto(BaseModel):
    subscriber_id: Annotated[str, pydantic.Field(alias="subscriberId")]
    r"""External unique identifier of the subscriber"""

    id: Annotated[str, pydantic.Field(alias="_id")]
    r"""Internal to Novu unique identifier of the subscriber"""

    first_name: Annotated[Optional[str], pydantic.Field(alias="firstName")] = None
    r"""First name of the subscriber"""

    last_name: Annotated[Optional[str], pydantic.Field(alias="lastName")] = None
    r"""Last name of the subscriber"""

    email: Optional[str] = None
    r"""Email address of the subscriber"""

    phone: Optional[str] = None
    r"""Phone number of the subscriber"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class ChatControlDtoTypedDict(TypedDict):
    skip: NotRequired[Dict[str, Any]]
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""
    body: NotRequired[str]
    r"""Content of the chat message."""


class ChatControlDto(BaseModel):
    skip: Optional[Dict[str, Any]] = None
    r"""JSONLogic filter conditions for conditionally skipping the step execution. Supports complex logical operations with AND, OR, and comparison operators. See https://jsonlogic.com/ for full typing reference."""

    body: Optional[str] = None
    r"""Content of the chat message."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutpreviewpayloaddto import (
    LayoutPreviewPayloadDto,
    LayoutPreviewPayloadDtoTypedDict,
)
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LayoutPreviewRequestDtoTypedDict(TypedDict):
    control_values: NotRequired[Dict[str, Any]]
    r"""Optional control values for layout preview"""
    preview_payload: NotRequired[LayoutPreviewPayloadDtoTypedDict]
    r"""Optional payload for layout preview"""


class LayoutPreviewRequestDto(BaseModel):
    control_values: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="controlValues")
    ] = None
    r"""Optional control values for layout preview"""

    preview_payload: Annotated[
        Optional[LayoutPreviewPayloadDto], pydantic.Field(alias="previewPayload")
    ] = None
    r"""Optional payload for layout preview"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ChatOrPushProviderEnum(str, Enum):
    r"""The provider identifier for the credentials"""

    SLACK = "slack"
    DISCORD = "discord"
    MSTEAMS = "msteams"
    MATTERMOST = "mattermost"
    RYVER = "ryver"
    ZULIP = "zulip"
    GRAFANA_ON_CALL = "grafana-on-call"
    GETSTREAM = "getstream"
    ROCKET_CHAT = "rocket-chat"
    WHATSAPP_BUSINESS = "whatsapp-business"
    CHAT_WEBHOOK = "chat-webhook"
    NOVU_SLACK = "novu-slack"
    FCM = "fcm"
    APNS = "apns"
    EXPO = "expo"
    ONE_SIGNAL = "one-signal"
    PUSHPAD = "pushpad"
    PUSH_WEBHOOK = "push-webhook"
    PUSHER_BEAMS = "pusher-beams"
    APPIO = "appio"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WeekDays(str, Enum):
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"
    SUNDAY = "sunday"


class Ordinal(str, Enum):
    ONE = "1"
    TWO = "2"
    THREE = "3"
    FOUR = "4"
    FIVE = "5"
    LAST = "last"


class OrdinalValue(str, Enum):
    DAY = "day"
    WEEKDAY = "weekday"
    WEEKEND = "weekend"
    SUNDAY = "sunday"
    MONDAY = "monday"
    TUESDAY = "tuesday"
    WEDNESDAY = "wednesday"
    THURSDAY = "thursday"
    FRIDAY = "friday"
    SATURDAY = "saturday"


class MonthlyType(str, Enum):
    EACH = "each"
    ON = "on"


class TimedConfigTypedDict(TypedDict):
    at_time: NotRequired[str]
    week_days: NotRequired[List[WeekDays]]
    month_days: NotRequired[List[str]]
    ordinal: NotRequired[Ordinal]
    ordinal_value: NotRequired[OrdinalValue]
    monthly_type: NotRequired[MonthlyType]


class TimedConfig(BaseModel):
    at_time: Annotated[Optional[str], pydantic.Field(alias="atTime")] = None

    week_days: Annotated[Optional[List[WeekDays]], pydantic.Field(alias="weekDays")] = (
        None
    )

    month_days: Annotated[Optional[List[str]], pydantic.Field(alias="monthDays")] = None

    ordinal: Optional[Ordinal] = None

    ordinal_value: Annotated[
        Optional[OrdinalValue], pydantic.Field(alias="ordinalValue")
    ] = None

    monthly_type: Annotated[
        Optional[MonthlyType], pydantic.Field(alias="monthlyType")
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, PathParamMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TranslationControllerDeleteTranslationEndpointPathParamResourceType(str, Enum):
    r"""Resource type"""

    WORKFLOW = "workflow"
    LAYOUT = "layout"


class TranslationControllerDeleteTranslationEndpointRequestTypedDict(TypedDict):
    resource_type: TranslationControllerDeleteTranslationEndpointPathParamResourceType
    r"""Resource type"""
    resource_id: str
    r"""Resource ID"""
    locale: str
    r"""Locale code"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class TranslationControllerDeleteTranslationEndpointRequest(BaseModel):
    resource_type: Annotated[
        TranslationControllerDeleteTranslationEndpointPathParamResourceType,
        pydantic.Field(alias="resourceType"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Resource type"""

    resource_id: Annotated[
        str,
        pydantic.Field(alias="resourceId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""Resource ID"""

    locale: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Locale code"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict
from typing_extensions import Annotated, TypedDict


class TranslationResponseDtoResourceType(str, Enum):
    r"""Resource type"""

    WORKFLOW = "workflow"
    LAYOUT = "layout"


class TranslationResponseDtoTypedDict(TypedDict):
    resource_id: str
    r"""Resource identifier"""
    resource_type: TranslationResponseDtoResourceType
    r"""Resource type"""
    locale: str
    r"""Locale code"""
    content: Dict[str, Any]
    r"""Translation content as JSON object"""
    created_at: str
    r"""Creation timestamp"""
    updated_at: str
    r"""Last update timestamp"""


class TranslationResponseDto(BaseModel):
    resource_id: Annotated[str, pydantic.Field(alias="resourceId")]
    r"""Resource identifier"""

    resource_type: Annotated[
        TranslationResponseDtoResourceType, pydantic.Field(alias="resourceType")
    ]
    r"""Resource type"""

    locale: str
    r"""Locale code"""

    content: Dict[str, Any]
    r"""Translation content as JSON object"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""Creation timestamp"""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""Last update timestamp"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listtopicsubscriptionsresponsedto import (
    ListTopicSubscriptionsResponseDto,
    ListTopicSubscriptionsResponseDtoTypedDict,
)
from enum import Enum
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscribersControllerListSubscriberTopicsQueryParamOrderDirection(str, Enum):
    r"""Direction of sorting"""

    ASC = "ASC"
    DESC = "DESC"


class SubscribersControllerListSubscriberTopicsRequestTypedDict(TypedDict):
    subscriber_id: str
    r"""The identifier of the subscriber"""
    after: NotRequired[str]
    r"""Cursor for pagination indicating the starting point after which to fetch results."""
    before: NotRequired[str]
    r"""Cursor for pagination indicating the ending point before which to fetch results."""
    limit: NotRequired[float]
    r"""Limit the number of items to return (max 100)"""
    order_direction: NotRequired[
        SubscribersControllerListSubscriberTopicsQueryParamOrderDirection
    ]
    r"""Direction of sorting"""
    order_by: NotRequired[str]
    r"""Field to order by"""
    include_cursor: NotRequired[bool]
    r"""Include cursor item in response"""
    key: NotRequired[str]
    r"""Filter by topic key"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class SubscribersControllerListSubscriberTopicsRequest(BaseModel):
    subscriber_id: Annotated[
        str,
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The identifier of the subscriber"""

    after: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Cursor for pagination indicating the starting point after which to fetch results."""

    before: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Cursor for pagination indicating the ending point before which to fetch results."""

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Limit the number of items to return (max 100)"""

    order_direction: Annotated[
        Optional[SubscribersControllerListSubscriberTopicsQueryParamOrderDirection],
        pydantic.Field(alias="orderDirection"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Direction of sorting"""

    order_by: Annotated[
        Optional[str],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Field to order by"""

    include_cursor: Annotated[
        Optional[bool],
        pydantic.Field(alias="includeCursor"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Include cursor item in response"""

    key: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter by topic key"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class SubscribersControllerListSubscriberTopicsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ListTopicSubscriptionsResponseDtoTypedDict


class SubscribersControllerListSubscriberTopicsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ListTopicSubscriptionsResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MessageStatusEnum(str, Enum):
    r"""Status of the message"""

    SENT = "sent"
    ERROR = "error"
    WARNING = "warning"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .emailcontrolsdto import EmailControlsDto, EmailControlsDtoTypedDict
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LayoutControlValuesDtoTypedDict(TypedDict):
    email: NotRequired[EmailControlsDtoTypedDict]
    r"""Email layout controls"""


class LayoutControlValuesDto(BaseModel):
    email: Optional[EmailControlsDto] = None
    r"""Email layout controls"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ContentIssueEnum(str, Enum):
    r"""Type of step content issue"""

    ILLEGAL_VARIABLE_IN_CONTROL_VALUE = "ILLEGAL_VARIABLE_IN_CONTROL_VALUE"
    INVALID_FILTER_ARG_IN_VARIABLE = "INVALID_FILTER_ARG_IN_VARIABLE"
    MISSING_VALUE = "MISSING_VALUE"
    TIER_LIMIT_EXCEEDED = "TIER_LIMIT_EXCEEDED"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .topicsubscriberdto import TopicSubscriberDto, TopicSubscriberDtoTypedDict
from novu_py.types import BaseModel
from novu_py.utils import FieldMetadata, HeaderMetadata, PathParamMetadata
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TopicsV1ControllerGetTopicSubscriberRequestTypedDict(TypedDict):
    external_subscriber_id: str
    r"""The external subscriber id"""
    topic_key: str
    r"""The topic key"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class TopicsV1ControllerGetTopicSubscriberRequest(BaseModel):
    external_subscriber_id: Annotated[
        str,
        pydantic.Field(alias="externalSubscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The external subscriber id"""

    topic_key: Annotated[
        str,
        pydantic.Field(alias="topicKey"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The topic key"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class TopicsV1ControllerGetTopicSubscriberResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: TopicSubscriberDtoTypedDict


class TopicsV1ControllerGetTopicSubscriberResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: TopicSubscriberDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class DelayScheduledMetadataType(str, Enum):
    SCHEDULED = "scheduled"


class DelayScheduledMetadataTypedDict(TypedDict):
    type: DelayScheduledMetadataType
    delay_path: str


class DelayScheduledMetadata(BaseModel):
    type: DelayScheduledMetadataType

    delay_path: Annotated[str, pydantic.Field(alias="delayPath")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .delaycontroldto import DelayControlDto, DelayControlDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


DelayStepUpsertDtoControlValuesTypedDict = TypeAliasType(
    "DelayStepUpsertDtoControlValuesTypedDict",
    Union[DelayControlDtoTypedDict, Dict[str, Any]],
)
r"""Control values for the Delay step."""


DelayStepUpsertDtoControlValues = TypeAliasType(
    "DelayStepUpsertDtoControlValues", Union[DelayControlDto, Dict[str, Any]]
)
r"""Control values for the Delay step."""


class DelayStepUpsertDtoTypedDict(TypedDict):
    name: str
    r"""Name of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    id: NotRequired[str]
    r"""Database identifier of the step. Used for updating the step."""
    step_id: NotRequired[str]
    r"""Unique identifier for the step"""
    control_values: NotRequired[DelayStepUpsertDtoControlValuesTypedDict]
    r"""Control values for the Delay step."""


class DelayStepUpsertDto(BaseModel):
    name: str
    r"""Name of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Database identifier of the step. Used for updating the step."""

    step_id: Annotated[Optional[str], pydantic.Field(alias="stepId")] = None
    r"""Unique identifier for the step"""

    control_values: Annotated[
        Optional[DelayStepUpsertDtoControlValues], pydantic.Field(alias="controlValues")
    ] = None
    r"""Control values for the Delay step."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class UpdateSubscriberOnlineFlagRequestDtoTypedDict(TypedDict):
    is_online: bool


class UpdateSubscriberOnlineFlagRequestDto(BaseModel):
    is_online: Annotated[bool, pydantic.Field(alias="isOnline")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .phoneendpointdto import PhoneEndpointDto, PhoneEndpointDtoTypedDict
from .slackchannelendpointdto import (
    SlackChannelEndpointDto,
    SlackChannelEndpointDtoTypedDict,
)
from .slackuserendpointdto import SlackUserEndpointDto, SlackUserEndpointDtoTypedDict
from .webhookendpointdto import WebhookEndpointDto, WebhookEndpointDtoTypedDict
from enum import Enum
from novu_py.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class GetChannelEndpointResponseDtoChannel(str, Enum):
    r"""The channel type (email, sms, push, chat, etc.)."""

    IN_APP = "in_app"
    EMAIL = "email"
    SMS = "sms"
    CHAT = "chat"
    PUSH = "push"


class GetChannelEndpointResponseDtoProviderID(str, Enum):
    r"""The provider identifier (e.g., sendgrid, twilio, slack, etc.)."""

    EMAILJS = "emailjs"
    MAILGUN = "mailgun"
    MAILJET = "mailjet"
    MANDRILL = "mandrill"
    NODEMAILER = "nodemailer"
    POSTMARK = "postmark"
    SENDGRID = "sendgrid"
    SENDINBLUE = "sendinblue"
    SES = "ses"
    NETCORE = "netcore"
    INFOBIP_EMAIL = "infobip-email"
    RESEND = "resend"
    PLUNK = "plunk"
    MAILERSEND = "mailersend"
    MAILTRAP = "mailtrap"
    CLICKATELL = "clickatell"
    OUTLOOK365 = "outlook365"
    NOVU_EMAIL = "novu-email"
    SPARKPOST = "sparkpost"
    EMAIL_WEBHOOK = "email-webhook"
    BRAZE = "braze"
    NEXMO = "nexmo"
    PLIVO = "plivo"
    SMS77 = "sms77"
    SMS_CENTRAL = "sms-central"
    SNS = "sns"
    TELNYX = "telnyx"
    TWILIO = "twilio"
    GUPSHUP = "gupshup"
    FIRETEXT = "firetext"
    INFOBIP_SMS = "infobip-sms"
    BURST_SMS = "burst-sms"
    BULK_SMS = "bulk-sms"
    ISEND_SMS = "isend-sms"
    FORTY_SIX_ELKS = "forty-six-elks"
    KANNEL = "kannel"
    MAQSAM = "maqsam"
    TERMII = "termii"
    AFRICAS_TALKING = "africas-talking"
    NOVU_SMS = "novu-sms"
    SENDCHAMP = "sendchamp"
    GENERIC_SMS = "generic-sms"
    CLICKSEND = "clicksend"
    BANDWIDTH = "bandwidth"
    MESSAGEBIRD = "messagebird"
    SIMPLETEXTING = "simpletexting"
    AZURE_SMS = "azure-sms"
    RING_CENTRAL = "ring-central"
    BREVO_SMS = "brevo-sms"
    EAZY_SMS = "eazy-sms"
    MOBISHASTRA = "mobishastra"
    AFRO_MESSAGE = "afro-message"
    UNIFONIC = "unifonic"
    SMSMODE = "smsmode"
    IMEDIA = "imedia"
    SINCH = "sinch"
    ISENDPRO_SMS = "isendpro-sms"
    FCM = "fcm"
    APNS = "apns"
    EXPO = "expo"
    ONE_SIGNAL = "one-signal"
    PUSHPAD = "pushpad"
    PUSH_WEBHOOK = "push-webhook"
    PUSHER_BEAMS = "pusher-beams"
    APPIO = "appio"
    NOVU = "novu"
    SLACK = "slack"
    DISCORD = "discord"
    MSTEAMS = "msteams"
    MATTERMOST = "mattermost"
    RYVER = "ryver"
    ZULIP = "zulip"
    GRAFANA_ON_CALL = "grafana-on-call"
    GETSTREAM = "getstream"
    ROCKET_CHAT = "rocket-chat"
    WHATSAPP_BUSINESS = "whatsapp-business"
    CHAT_WEBHOOK = "chat-webhook"
    NOVU_SLACK = "novu-slack"


class GetChannelEndpointResponseDtoType(str, Enum):
    r"""Type of channel endpoint"""

    SLACK_CHANNEL = "slack_channel"
    SLACK_USER = "slack_user"
    WEBHOOK = "webhook"
    PHONE = "phone"
    MS_TEAMS_CHANNEL = "ms_teams_channel"
    MS_TEAMS_USER = "ms_teams_user"


EndpointTypedDict = TypeAliasType(
    "EndpointTypedDict",
    Union[
        SlackChannelEndpointDtoTypedDict,
        SlackUserEndpointDtoTypedDict,
        PhoneEndpointDtoTypedDict,
        WebhookEndpointDtoTypedDict,
    ],
)
r"""Endpoint data specific to the channel type"""


Endpoint = TypeAliasType(
    "Endpoint",
    Union[
        SlackChannelEndpointDto,
        SlackUserEndpointDto,
        PhoneEndpointDto,
        WebhookEndpointDto,
    ],
)
r"""Endpoint data specific to the channel type"""


class GetChannelEndpointResponseDtoTypedDict(TypedDict):
    identifier: str
    r"""The unique identifier of the channel endpoint."""
    channel: Nullable[GetChannelEndpointResponseDtoChannel]
    r"""The channel type (email, sms, push, chat, etc.)."""
    provider_id: Nullable[GetChannelEndpointResponseDtoProviderID]
    r"""The provider identifier (e.g., sendgrid, twilio, slack, etc.)."""
    integration_identifier: Nullable[str]
    r"""The identifier of the integration to use for this channel endpoint."""
    connection_identifier: Nullable[str]
    r"""The identifier of the channel connection used for this endpoint."""
    subscriber_id: Nullable[str]
    r"""The subscriber ID to which the channel endpoint is linked"""
    context_keys: List[str]
    r"""The context of the channel connection"""
    type: GetChannelEndpointResponseDtoType
    r"""Type of channel endpoint"""
    endpoint: EndpointTypedDict
    r"""Endpoint data specific to the channel type"""
    created_at: str
    r"""The timestamp indicating when the channel endpoint was created, in ISO 8601 format."""
    updated_at: str
    r"""The timestamp indicating when the channel endpoint was last updated, in ISO 8601 format."""


class GetChannelEndpointResponseDto(BaseModel):
    identifier: str
    r"""The unique identifier of the channel endpoint."""

    channel: Nullable[GetChannelEndpointResponseDtoChannel]
    r"""The channel type (email, sms, push, chat, etc.)."""

    provider_id: Annotated[
        Nullable[GetChannelEndpointResponseDtoProviderID],
        pydantic.Field(alias="providerId"),
    ]
    r"""The provider identifier (e.g., sendgrid, twilio, slack, etc.)."""

    integration_identifier: Annotated[
        Nullable[str], pydantic.Field(alias="integrationIdentifier")
    ]
    r"""The identifier of the integration to use for this channel endpoint."""

    connection_identifier: Annotated[
        Nullable[str], pydantic.Field(alias="connectionIdentifier")
    ]
    r"""The identifier of the channel connection used for this endpoint."""

    subscriber_id: Annotated[Nullable[str], pydantic.Field(alias="subscriberId")]
    r"""The subscriber ID to which the channel endpoint is linked"""

    context_keys: Annotated[List[str], pydantic.Field(alias="contextKeys")]
    r"""The context of the channel connection"""

    type: GetChannelEndpointResponseDtoType
    r"""Type of channel endpoint"""

    endpoint: Endpoint
    r"""Endpoint data specific to the channel type"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The timestamp indicating when the channel endpoint was created, in ISO 8601 format."""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""The timestamp indicating when the channel endpoint was last updated, in ISO 8601 format."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "channel",
            "providerId",
            "integrationIdentifier",
            "connectionIdentifier",
            "subscriberId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ChannelCTATypeEnum(str, Enum):
    r"""Type of call to action"""

    REDIRECT = "redirect"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inappcontroldto import InAppControlDto, InAppControlDtoTypedDict
from .steptypeenum import StepTypeEnum
from novu_py.types import BaseModel
import pydantic
from typing import Any, Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


InAppStepUpsertDtoControlValuesTypedDict = TypeAliasType(
    "InAppStepUpsertDtoControlValuesTypedDict",
    Union[InAppControlDtoTypedDict, Dict[str, Any]],
)
r"""Control values for the In-App step."""


InAppStepUpsertDtoControlValues = TypeAliasType(
    "InAppStepUpsertDtoControlValues", Union[InAppControlDto, Dict[str, Any]]
)
r"""Control values for the In-App step."""


class InAppStepUpsertDtoTypedDict(TypedDict):
    name: str
    r"""Name of the step"""
    type: StepTypeEnum
    r"""Type of the step"""
    id: NotRequired[str]
    r"""Database identifier of the step. Used for updating the step."""
    step_id: NotRequired[str]
    r"""Unique identifier for the step"""
    control_values: NotRequired[InAppStepUpsertDtoControlValuesTypedDict]
    r"""Control values for the In-App step."""


class InAppStepUpsertDto(BaseModel):
    name: str
    r"""Name of the step"""

    type: StepTypeEnum
    r"""Type of the step"""

    id: Annotated[Optional[str], pydantic.Field(alias="_id")] = None
    r"""Database identifier of the step. Used for updating the step."""

    step_id: Annotated[Optional[str], pydantic.Field(alias="stepId")] = None
    r"""Unique identifier for the step"""

    control_values: Annotated[
        Optional[InAppStepUpsertDtoControlValues], pydantic.Field(alias="controlValues")
    ] = None
    r"""Control values for the In-App step."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Any, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InboundWebhooksControllerHandleWebhookRequestTypedDict(TypedDict):
    environment_id: str
    r"""The environment identifier"""
    integration_id: str
    r"""The integration identifier for the delivery provider"""
    request_body: Any
    r"""Webhook event payload from the delivery provider"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class InboundWebhooksControllerHandleWebhookRequest(BaseModel):
    environment_id: Annotated[
        str,
        pydantic.Field(alias="environmentId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The environment identifier"""

    integration_id: Annotated[
        str,
        pydantic.Field(alias="integrationId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The integration identifier for the delivery provider"""

    request_body: Annotated[
        Any, FieldMetadata(request=RequestMetadata(media_type="application/json"))
    ]
    r"""Webhook event payload from the delivery provider"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .layoutcreationsourceenum import LayoutCreationSourceEnum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateLayoutDtoTypedDict(TypedDict):
    layout_id: str
    r"""Unique identifier for the layout"""
    name: str
    r"""Name of the layout"""
    is_translation_enabled: NotRequired[bool]
    r"""Enable or disable translations for this layout"""
    source: NotRequired[LayoutCreationSourceEnum]
    r"""Source of layout creation"""


class CreateLayoutDto(BaseModel):
    layout_id: Annotated[str, pydantic.Field(alias="layoutId")]
    r"""Unique identifier for the layout"""

    name: str
    r"""Name of the layout"""

    is_translation_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="isTranslationEnabled")
    ] = False
    r"""Enable or disable translations for this layout"""

    source: Annotated[
        Optional[LayoutCreationSourceEnum], pydantic.Field(alias="__source")
    ] = LayoutCreationSourceEnum.DASHBOARD
    r"""Source of layout creation"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .feedresponsedto import FeedResponseDto, FeedResponseDtoTypedDict
from novu_py.types import BaseModel
from novu_py.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SubscribersV1ControllerGetNotificationsFeedRequestTypedDict(TypedDict):
    subscriber_id: str
    page: NotRequired[float]
    limit: NotRequired[float]
    read: NotRequired[bool]
    seen: NotRequired[bool]
    payload: NotRequired[str]
    r"""Base64 encoded string of the partial payload JSON object"""
    idempotency_key: NotRequired[str]
    r"""A header for idempotency purposes"""


class SubscribersV1ControllerGetNotificationsFeedRequest(BaseModel):
    subscriber_id: Annotated[
        str,
        pydantic.Field(alias="subscriberId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    page: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    limit: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10

    read: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    seen: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    payload: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Base64 encoded string of the partial payload JSON object"""

    idempotency_key: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A header for idempotency purposes"""


class SubscribersV1ControllerGetNotificationsFeedResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: FeedResponseDtoTypedDict


class SubscribersV1ControllerGetNotificationsFeedResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: FeedResponseDto



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .builderfieldtypeenum import BuilderFieldTypeEnum
from .fieldfilterpartdto import FieldFilterPartDto, FieldFilterPartDtoTypedDict
from enum import Enum
from novu_py.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class StepFilterDtoValue(str, Enum):
    AND = "AND"
    OR = "OR"


class StepFilterDtoTypedDict(TypedDict):
    is_negated: bool
    type: BuilderFieldTypeEnum
    value: StepFilterDtoValue
    children: List[FieldFilterPartDtoTypedDict]


class StepFilterDto(BaseModel):
    is_negated: Annotated[bool, pydantic.Field(alias="isNegated")]

    type: BuilderFieldTypeEnum

    value: StepFilterDtoValue

    children: List[FieldFilterPartDto]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .phoneendpointdto import PhoneEndpointDto, PhoneEndpointDtoTypedDict
from .slackchannelendpointdto import (
    SlackChannelEndpointDto,
    SlackChannelEndpointDtoTypedDict,
)
from .slackuserendpointdto import SlackUserEndpointDto, SlackUserEndpointDtoTypedDict
from .webhookendpointdto import WebhookEndpointDto, WebhookEndpointDtoTypedDict
from novu_py.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


UpdateChannelEndpointRequestDtoEndpointTypedDict = TypeAliasType(
    "UpdateChannelEndpointRequestDtoEndpointTypedDict",
    Union[
        SlackChannelEndpointDtoTypedDict,
        SlackUserEndpointDtoTypedDict,
        PhoneEndpointDtoTypedDict,
        WebhookEndpointDtoTypedDict,
    ],
)
r"""Updated endpoint data. The structure must match the existing channel endpoint type."""


UpdateChannelEndpointRequestDtoEndpoint = TypeAliasType(
    "UpdateChannelEndpointRequestDtoEndpoint",
    Union[
        SlackChannelEndpointDto,
        SlackUserEndpointDto,
        PhoneEndpointDto,
        WebhookEndpointDto,
    ],
)
r"""Updated endpoint data. The structure must match the existing channel endpoint type."""


class UpdateChannelEndpointRequestDtoTypedDict(TypedDict):
    endpoint: UpdateChannelEndpointRequestDtoEndpointTypedDict
    r"""Updated endpoint data. The structure must match the existing channel endpoint type."""


class UpdateChannelEndpointRequestDto(BaseModel):
    endpoint: UpdateChannelEndpointRequestDtoEndpoint
    r"""Updated endpoint data. The structure must match the existing channel endpoint type."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .channelcredentials import ChannelCredentials, ChannelCredentialsTypedDict
from .chatorpushproviderenum import ChatOrPushProviderEnum
from novu_py.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateSubscriberChannelRequestDtoTypedDict(TypedDict):
    provider_id: ChatOrPushProviderEnum
    r"""The provider identifier for the credentials"""
    credentials: ChannelCredentialsTypedDict
    r"""Credentials payload for the specified provider"""
    integration_identifier: NotRequired[str]
    r"""The integration identifier"""


class UpdateSubscriberChannelRequestDto(BaseModel):
    provider_id: Annotated[ChatOrPushProviderEnum, pydantic.Field(alias="providerId")]
    r"""The provider identifier for the credentials"""

    credentials: ChannelCredentials
    r"""Credentials payload for the specified provider"""

    integration_identifier: Annotated[
        Optional[str], pydantic.Field(alias="integrationIdentifier")
    ] = None
    r"""The integration identifier"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
from typing_extensions import TypedDict


class EmailLayoutRenderOutputTypedDict(TypedDict):
    body: str
    r"""Content of the email"""


class EmailLayoutRenderOutput(BaseModel):
    body: str
    r"""Content of the email"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from novu_py import models, utils
from novu_py._hooks import HookContext
from novu_py.types import BaseModel, OptionalNullable, UNSET
from novu_py.utils import get_security_from_env
from novu_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class Subscriptions(BaseSDK):
    def list(
        self,
        *,
        request: Union[
            models.TopicsControllerListTopicSubscriptionsRequest,
            models.TopicsControllerListTopicSubscriptionsRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerListTopicSubscriptionsResponse:
        r"""List topic subscriptions

        List all subscriptions of subscribers for a topic.
        Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.TopicsControllerListTopicSubscriptionsRequest
            )
        request = cast(models.TopicsControllerListTopicSubscriptionsRequest, request)

        req = self._build_request(
            method="GET",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_listTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerListTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.ListTopicSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        request: Union[
            models.TopicsControllerListTopicSubscriptionsRequest,
            models.TopicsControllerListTopicSubscriptionsRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerListTopicSubscriptionsResponse:
        r"""List topic subscriptions

        List all subscriptions of subscribers for a topic.
        Checkout all available filters in the query section.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.TopicsControllerListTopicSubscriptionsRequest
            )
        request = cast(models.TopicsControllerListTopicSubscriptionsRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_listTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerListTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.ListTopicSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        topic_key: str,
        create_topic_subscriptions_request_dto: Union[
            models.CreateTopicSubscriptionsRequestDto,
            models.CreateTopicSubscriptionsRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerCreateTopicSubscriptionsResponse:
        r"""Create topic subscriptions

        This api will create subscription for subscriberIds for a topic.
        Its like subscribing to a common interest group. if topic does not exist, it will be created.

        :param topic_key: The key identifier of the topic
        :param create_topic_subscriptions_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerCreateTopicSubscriptionsRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            create_topic_subscriptions_request_dto=utils.get_pydantic_model(
                create_topic_subscriptions_request_dto,
                models.CreateTopicSubscriptionsRequestDto,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_topic_subscriptions_request_dto,
                False,
                False,
                "json",
                models.CreateTopicSubscriptionsRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_createTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.TopicsControllerCreateTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.CreateSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        topic_key: str,
        create_topic_subscriptions_request_dto: Union[
            models.CreateTopicSubscriptionsRequestDto,
            models.CreateTopicSubscriptionsRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerCreateTopicSubscriptionsResponse:
        r"""Create topic subscriptions

        This api will create subscription for subscriberIds for a topic.
        Its like subscribing to a common interest group. if topic does not exist, it will be created.

        :param topic_key: The key identifier of the topic
        :param create_topic_subscriptions_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerCreateTopicSubscriptionsRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            create_topic_subscriptions_request_dto=utils.get_pydantic_model(
                create_topic_subscriptions_request_dto,
                models.CreateTopicSubscriptionsRequestDto,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_topic_subscriptions_request_dto,
                False,
                False,
                "json",
                models.CreateTopicSubscriptionsRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_createTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.TopicsControllerCreateTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.CreateSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        topic_key: str,
        delete_topic_subscriptions_request_dto: Union[
            models.DeleteTopicSubscriptionsRequestDto,
            models.DeleteTopicSubscriptionsRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerDeleteTopicSubscriptionsResponse:
        r"""Delete topic subscriptions

        Delete subscriptions for subscriberIds for a topic.

        :param topic_key: The key identifier of the topic
        :param delete_topic_subscriptions_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerDeleteTopicSubscriptionsRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            delete_topic_subscriptions_request_dto=utils.get_pydantic_model(
                delete_topic_subscriptions_request_dto,
                models.DeleteTopicSubscriptionsRequestDto,
            ),
        )

        req = self._build_request(
            method="DELETE",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.delete_topic_subscriptions_request_dto,
                False,
                False,
                "json",
                models.DeleteTopicSubscriptionsRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_deleteTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerDeleteTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.DeleteTopicSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        topic_key: str,
        delete_topic_subscriptions_request_dto: Union[
            models.DeleteTopicSubscriptionsRequestDto,
            models.DeleteTopicSubscriptionsRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerDeleteTopicSubscriptionsResponse:
        r"""Delete topic subscriptions

        Delete subscriptions for subscriberIds for a topic.

        :param topic_key: The key identifier of the topic
        :param delete_topic_subscriptions_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerDeleteTopicSubscriptionsRequest(
            topic_key=topic_key,
            idempotency_key=idempotency_key,
            delete_topic_subscriptions_request_dto=utils.get_pydantic_model(
                delete_topic_subscriptions_request_dto,
                models.DeleteTopicSubscriptionsRequestDto,
            ),
        )

        req = self._build_request_async(
            method="DELETE",
            path="/v2/topics/{topicKey}/subscriptions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.delete_topic_subscriptions_request_dto,
                False,
                False,
                "json",
                models.DeleteTopicSubscriptionsRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_deleteTopicSubscriptions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerDeleteTopicSubscriptionsResponse(
                result=unmarshal_json_response(
                    models.DeleteTopicSubscriptionsResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def get_subscription(
        self,
        *,
        topic_key: str,
        identifier: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerGetTopicSubscriptionResponse:
        r"""Get a topic subscription

        Get a subscription by its unique identifier for a topic.

        :param topic_key: The key identifier of the topic
        :param identifier: The unique identifier of the subscription
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerGetTopicSubscriptionRequest(
            topic_key=topic_key,
            identifier=identifier,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/v2/topics/{topicKey}/subscriptions/{identifier}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_getTopicSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerGetTopicSubscriptionResponse(
                result=unmarshal_json_response(
                    models.SubscriptionResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_subscription_async(
        self,
        *,
        topic_key: str,
        identifier: str,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerGetTopicSubscriptionResponse:
        r"""Get a topic subscription

        Get a subscription by its unique identifier for a topic.

        :param topic_key: The key identifier of the topic
        :param identifier: The unique identifier of the subscription
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerGetTopicSubscriptionRequest(
            topic_key=topic_key,
            identifier=identifier,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/v2/topics/{topicKey}/subscriptions/{identifier}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_getTopicSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerGetTopicSubscriptionResponse(
                result=unmarshal_json_response(
                    models.SubscriptionResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        topic_key: str,
        identifier: str,
        update_topic_subscription_request_dto: Union[
            models.UpdateTopicSubscriptionRequestDto,
            models.UpdateTopicSubscriptionRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpdateTopicSubscriptionResponse:
        r"""Update a topic subscription

        Update a subscription by its unique identifier for a topic. You can update the preferences and name associated with the subscription.

        :param topic_key: The key identifier of the topic
        :param identifier: The unique identifier of the subscription
        :param update_topic_subscription_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpdateTopicSubscriptionRequest(
            topic_key=topic_key,
            identifier=identifier,
            idempotency_key=idempotency_key,
            update_topic_subscription_request_dto=utils.get_pydantic_model(
                update_topic_subscription_request_dto,
                models.UpdateTopicSubscriptionRequestDto,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v2/topics/{topicKey}/subscriptions/{identifier}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_topic_subscription_request_dto,
                False,
                False,
                "json",
                models.UpdateTopicSubscriptionRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_updateTopicSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerUpdateTopicSubscriptionResponse(
                result=unmarshal_json_response(
                    models.SubscriptionResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        topic_key: str,
        identifier: str,
        update_topic_subscription_request_dto: Union[
            models.UpdateTopicSubscriptionRequestDto,
            models.UpdateTopicSubscriptionRequestDtoTypedDict,
        ],
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TopicsControllerUpdateTopicSubscriptionResponse:
        r"""Update a topic subscription

        Update a subscription by its unique identifier for a topic. You can update the preferences and name associated with the subscription.

        :param topic_key: The key identifier of the topic
        :param identifier: The unique identifier of the subscription
        :param update_topic_subscription_request_dto:
        :param idempotency_key: A header for idempotency purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TopicsControllerUpdateTopicSubscriptionRequest(
            topic_key=topic_key,
            identifier=identifier,
            idempotency_key=idempotency_key,
            update_topic_subscription_request_dto=utils.get_pydantic_model(
                update_topic_subscription_request_dto,
                models.UpdateTopicSubscriptionRequestDto,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v2/topics/{topicKey}/subscriptions/{identifier}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_topic_subscription_request_dto,
                False,
                False,
                "json",
                models.UpdateTopicSubscriptionRequestDto,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(1000, 30000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "409", "429", "5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="TopicsController_updateTopicSubscription",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "413",
                "414",
                "415",
                "422",
                "429",
                "4XX",
                "500",
                "503",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.TopicsControllerUpdateTopicSubscriptionResponse(
                result=unmarshal_json_response(
                    models.SubscriptionResponseDto, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "414", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(
            http_res,
            ["400", "401", "403", "404", "405", "409", "413", "415"],
            "application/json",
        ):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.ValidationErrorDtoData, http_res
            )
            raise models.ValidationErrorDto(response_data, http_res)
        if utils.match_response(http_res, "429", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(models.ErrorDtoData, http_res)
            raise models.ErrorDto(response_data, http_res)
        if utils.match_response(http_res, "503", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriberresponsedtooptional import (
    SubscriberResponseDtoOptional,
    SubscriberResponseDtoOptionalTypedDict,
)
from novu_py.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LayoutPreviewPayloadDtoTypedDict(TypedDict):
    subscriber: NotRequired[SubscriberResponseDtoOptionalTypedDict]
    r"""Partial subscriber information"""


class LayoutPreviewPayloadDto(BaseModel):
    subscriber: Optional[SubscriberResponseDtoOptional] = None
    r"""Partial subscriber information"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from novu_py.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PhoneEndpointDtoTypedDict(TypedDict):
    phone_number: str
    r"""Phone number in E.164 format"""


class PhoneEndpointDto(BaseModel):
    phone_number: Annotated[str, pydantic.Field(alias="phoneNumber")]
    r"""Phone number in E.164 format"""
